/***************************************************************************

    M.A.M.E.CE  -  Multiple Arcade Machine Emulator for WinCE
    Win32 Portions Copyright (C) 1997-98 Michael Soderstrom and Chris Kirmse
	WinCE Portions Copyright (C) 1999 Benjamin Cooley
    
    This file is part of MAMECE, and may only be used, modified and
    distributed under the terms of the MAME license, in "readme.txt".
    By continuing to use, modify or distribute this file you indicate
    that you have read the license and understand and accept it fully.

 ***************************************************************************/

/***************************************************************************

  CESound.c
//This file was modified from its original version for use in PocketVCS
// by Stuart Russell

 ***************************************************************************/
#include "stdafx.h"
#include "mmSystem.h"
#include "CESound.h"
#include "tiasound.h"
#include "Atari2600EmulatorGlobals.h"

/***************************************************************************
    Internal structures
 ***************************************************************************/

#define NUM_WAVEHDRS  4

struct tSound_private {
	WAVEOUTCAPS		m_Caps;
	HWAVEOUT		m_hWaveOut;
	WAVEHDR			m_WaveHdrs[NUM_WAVEHDRS];

	int				m_nSampleRate;
	int				m_nSampleBits;
	int				m_nChannels;
	int				m_nVolume;		// -32 to 0 attenuation value
};


/***************************************************************************
    Internal variables
 ***************************************************************************/

static struct tSound_private      This;

/* Could set this up as an option... */

//#define AUDIO_CLOCK 34100

unsigned char *sounddata;

int CESound_init(void)
{
	MMRESULT res;
	WAVEFORMATEX wf;

	This.m_hWaveOut = NULL;

	if (nOptions_SoundSampleRate ==1) 
		This.m_nSampleRate = 11025;
	else if (nOptions_SoundSampleRate ==2) 
		This.m_nSampleRate = 22050;
	else if (nOptions_SoundSampleRate ==3)
		This.m_nSampleRate = 44100;

	This.m_nSampleBits = 8;
	This.m_nChannels = 1;

	sounddata = (unsigned char*) (malloc(5000));

	wf.wFormatTag = WAVE_FORMAT_PCM;
	wf.nChannels = This.m_nChannels; 
	wf.nSamplesPerSec = This.m_nSampleRate; 
	wf.nAvgBytesPerSec = This.m_nSampleRate * This.m_nSampleBits / 8; 
	wf.nBlockAlign = This.m_nSampleBits * This.m_nChannels / 8;
	wf.wBitsPerSample = This.m_nSampleBits; 
	wf.cbSize = 0;

	res = waveOutOpen(
		&This.m_hWaveOut,	// Handle
		0,					// ID (0 for wave mapper)
		&wf,				// Wave format
		0,					// Callback
		0,					// Instance data
		CALLBACK_NULL);

	if (res != MMSYSERR_NOERROR)
		return 1;

	memset(&This.m_WaveHdrs, 0, sizeof(WAVEHDR) * NUM_WAVEHDRS);

	This.m_nVolume = 0;

	//new
	Tia_sound_init((nOptions_SoundCreateRate*100), This.m_nSampleRate); 

	return 0;
}

void CESound_exit(void)
{
	BOOL done;
	int i, ticks;

	ticks = GetTickCount();
	done = FALSE;
	while (!done && GetTickCount() - ticks < 500)
	{
		done = TRUE;
		for (i = 0; i < NUM_WAVEHDRS; i++)
		{
			if (This.m_WaveHdrs[i].dwFlags & WHDR_DONE)
			{
				waveOutUnprepareHeader(This.m_hWaveOut, &This.m_WaveHdrs[i], sizeof(WAVEHDR));
				This.m_WaveHdrs[i].lpData = NULL;
				This.m_WaveHdrs[i].dwBufferLength = 0;
				This.m_WaveHdrs[i].dwFlags = 0;
			}
			else if (This.m_WaveHdrs[i].dwFlags)
				done = FALSE;
		}
	}

	waveOutClose(This.m_hWaveOut);
}

void CESound_set_volume(unsigned short left, unsigned short right)
{
	DWORD nVolume;

	nVolume = (DWORD)left | (DWORD)right << 16;

	waveOutSetVolume(This.m_hWaveOut, nVolume);
}

void CESound_play_sample(char *data, int len)
{
	int i;
	char *s;
	char *d;
	DWORD pos;

	for (i = 0; i < NUM_WAVEHDRS; i++)
	{
		if (This.m_WaveHdrs[i].dwFlags == 0 || This.m_WaveHdrs[i].dwFlags & WHDR_DONE)
		{
			if (This.m_WaveHdrs[i].lpData && This.m_WaveHdrs[i].dwBufferLength != len)
			{
				waveOutUnprepareHeader(This.m_hWaveOut, &This.m_WaveHdrs[i], sizeof(WAVEHDR));
				free(This.m_WaveHdrs[i].lpData);
				This.m_WaveHdrs[i].lpData = NULL;
			}
			if (!This.m_WaveHdrs[i].lpData)
			{
				This.m_WaveHdrs[i].lpData = (LPSTR)malloc(len);
				This.m_WaveHdrs[i].dwBufferLength = len;
				This.m_WaveHdrs[i].dwFlags = 0;
				waveOutPrepareHeader(This.m_hWaveOut, &This.m_WaveHdrs[i], sizeof(WAVEHDR));
			}
			s = data;
			d = (char *)This.m_WaveHdrs[i].lpData;
			
			for (pos = 0; pos < len; pos++)
				*d++ = *s++;

			waveOutWrite(This.m_hWaveOut, &This.m_WaveHdrs[i], sizeof(WAVEHDR));
			return;	
		}
	}
}

