/*****************************************************************************

   This file is part of Virtual VCS, the Atari 2600 Emulator
   ===================================================

   Copyright 1996 Alex Hornby. For contributions see the file CREDITS.
   Modified 1996 by Daniel Boris
   Modified 2001 by Stuart Russell

   This software is distributed under the terms of the GNU General Public
   License. This is free software with ABSOLUTELY NO WARRANTY.

   See the file COPYING for Details.

   $Id: display.c,v 1.23 1996/03/21 15:52:38 alex Exp $
******************************************************************************/
//This file was modified from its original version for use in PocketVCS
// by Stuart Russell

/*
   Display handling code.
*/

#include <stdio.h>
#include <stdlib.h>
#include "stdafx.h"
#include "types.h"
#include "btypes.h"
#include "vmachine.h"
#include "address.h"
#include "files.h"
#include "colours.h"
#include "keyboard.h"
#include "resource.h"
#include "Display.h"

#define VGA256		0x13
#define TEXT_MODE	0x03
#define PALETTE_MASK      0x3c6
#define PALETTE_REGISTER  0x3c8
#define PALETTE_DATA	  0x3c9

unsigned char far *screen=(char far *)0xA0000000L;
extern unsigned short color_palette[256];

unsigned long white,black;
int vwidth,vheight,theight;

/* The width and height of the buffer */
//int vwidth,vheight;

char coltable[256];

/* The refresh skipping counter */
int tv_counter=0;
int redraw_flag=1;



void SetPalette(unsigned char index, unsigned char r, unsigned char g, unsigned char b)
{
	color_palette[index] = (r * 2048) + (g * 32) + b;

}

/* Set up the colormap - original is 24bit!
//------------------------------------------------------------------------------------
// we use a 16 bit color palette (rrrrrggggggbbbbb). This should change based
// on the PDA device (E-125).
//------------------------------------------------------------------------------------*/
void create_cmap(void)
{
	int i;
	unsigned char red,green,blue;

	/* Initialise parts of the colors array */
	for(i=0; i< 256; i++)
	{
		red  = (unsigned char)((colortable[i] & 0xff0000) >> 16);
		green = (unsigned char)((colortable[i] & 0x00ff00) >> 8);
		blue = (unsigned char)(colortable[i] & 0x0000ff);
		red=red/8;
		green=green/4;
		blue=blue/8;
		SetPalette(i,red,green,blue);
	}
}


/* Create the main application shell */
static void create_window()
{
	int i;
	  /* Calculate the video width and height */
	//vwidth = base_opts.magstep * tv_width * 2;
	theight = tv_height + 16;//8
	vheight = theight;

	vwidth=tv_width;
	//vheight=tv_height+tv_overscan; // calculates to 192+30 = 222
	create_cmap();
	for(i=0; i<256; i++) coltable[i]=1;
	coltable[0]=coltable[1]=coltable[2]=coltable[4]=0;
	coltable[8]=coltable[16]=coltable[32]=coltable[64]=0;
	coltable[128]=0;
}

uint8 *XBuf;

/* The main initialiser for the X stuff */
int tv_on()
{
   /* Get the basic colors */
	unsigned long m;
	white=0;
	black=15;
	create_window();
	XBuf = (uint8*) (malloc((160+16) * (480+16) + 8));
	if(XBuf==NULL)
	{
		//Could not create buffer.
		return (0);
	}
	
	// if pointer size is 4 bytes, make sure the
	// buffer is aligned on an 8 byte boundary.
	if(sizeof(uint8*)==4)
	{
		m=(unsigned long) XBuf;
		m+=8;
		m&=0xFFFFFFF8;
		(unsigned long)XBuf=m;
	} 
	putc('\n',stdout);
	memset(XBuf,128,272*256);
	return(1);
}







