

/*****************************************************************************

   This file is part of x2600, the Atari 2600 Emulator
   ===================================================
   
   Copyright 1996 Alex Hornby. For contributions see the file CREDITS.

   This software is distributed under the terms of the GNU General Public
   License. This is free software with ABSOLUTELY NO WARRANTY.
   
   See the file COPYING for details.
   
   $Id: files.c,v 1.18 1997/02/23 20:39:29 ahornby Exp ahornby $
******************************************************************************/
//This file was modified from its original version for use in PocketVCS
// by Stuart Russell

/*
   Used to load cartridge images into memory.
 */

//#include "config.h"
#include <stdio.h>
//#include <unistd.h>
#include <string.h>
#include "options.h"
#include "types.h"
#include "vmachine.h"
#include "c26def.h"
//#include "okie.h"
//#include "dbg_mess.h"
#include "resource.h"
extern char filename[80];

/* Return the size of the file pointed to by fp */
/* Avoids need for UNIX type stat, leaves file */
/* pointer untouched. */
long
filesize (FILE * fp)
{
  long curpos, length;

  curpos = ftell (fp);
  fseek (fp, 0L, SEEK_END);
  length = ftell (fp);
  fseek (fp, curpos, SEEK_SET);
  return length;
}

/* Load from the common 2600 format */
/* Uses the tag structure from c26def.h */
int
loadc26 (FILE * fp, long flen)
{
  char databuf[65535];
  struct c26_tag tag;

  while (fread (&tag, sizeof (struct c26_tag), 1, fp))
    {
      /* If positive then it has a data block */
      if (tag.type >= 0)
	{
	  if (fread (databuf, sizeof (BYTE), tag.len, fp))
	    {
	      //perror ("v2600 loadc26:");
	     // fprintf (stderr, "Error reading data.\n");
	    }
	}

      switch (tag.type)
	{

	case VERSION:
	  //if (Verbose)
	  // printf ("File version %d\n", tag.len);
	  break;

	case WRITER:
	  //if (Verbose)
	  //  printf ("Written by: %s\n", databuf);
	  break;

	case TVTYPE:
	  base_opts.tvtype = tag.len;
	  break;

	case CONTROLLERS:
	  /* Higher byte */
	  base_opts.lcon = (tag.len & 0xff00)>>8;
	  /* Low byte */
	  base_opts.rcon = (tag.len) & 0xff;

	case BANKING:
	  base_opts.bank = tag.len;
	  break;

	case DATA:
	  if (tag.len <= 16384)
	    memcpy (&theCart[0], databuf, tag.len);
	  else
	    {
	      //fprintf (stderr, "Data larger that 16k!\n");
	      exit (-1);
	    }
	  rom_size = tag.len;
	  if (tag.len == 2048)
	    {
	      memcpy (&theCart[0], databuf, tag.len);
	      memcpy (&theCart[2048], databuf, tag.len);
	    }
	  break;

	default:
	  //fprintf (stderr, "Unknown tag %d\n. Ignoring.", tag.type);
	  break;
	}
    }
  return 0;
}

/* Load a raw binary image  */
/* fp: file stream to load */
/* stat_data: unix stat structure for file pointed to by fp */
/* returns: size of file loaded */
int
loadRaw (FILE * fp, long flen)
{
	int size = flen;

	if (size > 16384)
    size = 16384;
	
	fread (theCart, sizeof (BYTE), size, fp);

  rom_size = size;
  if (size == 2048)
    {
      memcpy (&theCart[2048], &theCart[0], 2048);
      rom_size = 4096;
    }
  else if (size < 2048)
    {
      theCart[0x0ffc] = 0x00;
      theCart[0x0ffd] = 0xf0;
      rom_size = 4096;
    }
  
  return size;
}

/* Load a commodore format .prg file  */
/* fp: file stream to load */
/* stat_data: unix stat structure for file pointed to by fp */
/* returns: size of file loaded */
int
loadPrg (FILE * fp, long flen)
{
  int start;
  int rompos;
  int size;
  unsigned char buf1, buf2;

  /* Get the load address */
  fread (&buf1, sizeof (BYTE), 1, fp);
  fread (&buf2, sizeof (BYTE), 1, fp);
  start = buf2 * 256 + buf1;

  /* Translate the load address to a ROM address */
  rompos = start & 0xfff;

  /* Get length of data part */
  size = flen - 2;

  //if (Verbose)
  //  printf ("Loading .prg at %x\n", start);

  /* Load the data */
  fread (&theCart[rompos], sizeof (BYTE), size, fp);

  /* Put the load address into the reset vector */
  theCart[0x0ffc] = buf1;
  theCart[0x0ffd] = buf2;
  return size;
}

// This routine was made to help with .bin file configurations. The compares are in HEX for recognition, although decimal
// Would have been easier to program... :-)
void autoconfig(unsigned long sum,int len)
{
	char ch[10];
	sprintf(ch,"%X",sum);// convert checksum to hex!

    if ((strcmp(ch,"3D708")==0) || (strcmp(ch , "7E2EA")==0) || (strcmp(ch , "7B217")==0) || (strcmp(ch , "35E7E")==0) || (strcmp(ch , "7CBE1")==0) || (strcmp(ch , "77DC4")==0) || (strcmp(ch , "72F18")==0) || (strcmp(ch , "6EAA5")==0) || (strcmp(ch , "6AE62")==0) || (strcmp(ch , "3C030")==0) || (strcmp(ch , "6D7FC")==0) || (strcmp(ch , "79F28")==0) || (strcmp(ch , "36C7C")==0) || (strcmp(ch , "70434")==0) || (strcmp(ch , "66E29")==0) || (strcmp(ch , "6CCD7")==0) || (strcmp(ch , "6EE82")==0) || (strcmp(ch , "7E271")==0) || (strcmp(ch , "5485D")==0) || (strcmp(ch , "54E06")==0) || (strcmp(ch , "3B06F")==0) || (strcmp(ch , "62E00")==0) || (strcmp(ch , "81AEF")==0))
	{
		base_opts.rcon=PADDLE;
		base_opts.lcon=PADDLE;
	}
	else{
		base_opts.rcon=STICK;
		base_opts.lcon=STICK;
	}
	//Eventually...
		//consoleright=KEYPAD;
		//consoleleft=KEYPAD;
    
	//Now we set up the mapping scheme for the cartridge based on size:
		if (len < 4000 )
	        //typ = "2K";
			base_opts.bank=0;
		else if (len < 8000 )
            //typ = "4K";
			base_opts.bank=0;
		else if (len < 12000 )
            //typ = "F8";
			base_opts.bank=1;
		else if (len< 16000 )
            //typ = "FA"; = CBS
			base_opts.bank=4;
		else if (len < 32000 )
            //typ = "F6";
			base_opts.bank=2;
		else if (len < 64000 )
			//typ = "AA"
			base_opts.bank=0;
		else
            //typ = "F6SC";//supercharger
			base_opts.bank=5;
	//Special Rom Mappings:
	if ((strcmp(ch,"CF815")==0) || (strcmp(ch , "DAD52")==0))
            //typ = "FE";//not yet supported
			base_opts.bank=0;
	if ((strcmp(ch,"DE446")==0) || (strcmp(ch,"EFCE0")==0) || (strcmp(ch,"EE31A")==0) || (strcmp(ch,"D5653")==0) || (strcmp(ch,"CCE17")==0) || (strcmp(ch,"CC026")==0) || (strcmp(ch,"DC831")==0) || (strcmp(ch,"109A6D")==0) || (strcmp(ch , "BF9F8")==0))
            //typ = "E0";//Parker Brothers!
			base_opts.bank=3;
	if ((strcmp(ch,"1694A9")==0) || (strcmp(ch,"2F03C8")==0) || (strcmp(ch , "188504")==0))
            //typ = "E7";//not yet supported
			base_opts.bank=0;
	
}


/* Load a cartridge image */
/* name: filename to load */
/* returns: -1 on error 0 otherwise  */
int
loadCart (void)
{
  FILE *fp;
  char *ext;
  int flen;
  unsigned long checksum;
  int i;
  char *name = filename;

  fp = fopen ( name , "rb");
  if(!fp) {
      //fprintf (stderr, "Can't find %s\n", name);
      return -1;
  }

  //if (Verbose)
  //  printf ("Loading cart: %s\n", name);
  flen = filesize (fp);

  ext = strrchr (name, '.');
  if(!ext) {
      ext=".bin";
  }
  
  if (strcmp (".pal", ext) == 0 || strcmp (".vcs", ext) == 0 ||
      strcmp (".raw", ext) == 0 || strcmp (".bin", ext) == 0 ||
      strcmp (".BIN", ext) == 0)
    {
	// Raw format data file, the most common.... :-)
	
	int len=loadRaw (fp, flen);
	checksum=0;
	for(i=0; i<len; i++) checksum+=theCart[i];
	autoconfig(checksum,len);// This sends a checksum of the ROM to the Autoconfigure routine...
	  
    }
  else if (strcmp (".prg", ext) == 0)
    {
      loadPrg (fp, flen);
    }
  else if (strcmp (".c26", ext) == 0)
    {
      loadc26 (fp, flen);
    }
  else
    {
      fprintf (stderr, "Unknown file format %s\n", ext);
      return -1;
    }
  fclose (fp);
  return 0;
}
