// Atari2600Emulator.cpp : Defines the entry point for the application.
// This program was written by Stuart Russell, with the generous contributions of
// Dan Boris (VVCS 0.60), and Kevin Buch (NesCE). Modifications are welcome, as long as
// credit is given to those who have contributed thus far. Please contact stu.russell@home.com for updates

#include "StdAfx.h"
#include "Atari2600Emulator.h"
#include <commctrl.h>
#include <aygshell.h>
#include <sipapi.h>
#include "Atari2600EmulatorGlobals.h"
#include <commdlg.h>
#include <stdlib.h>
#include "options.h"
#include "keyscan.h"
#include "display.h"
#include "gx.h"
#include "vmachine.h"


//E-125 keycodes (taken from NesCE)
#define VK_CASSIOPEIA_SLIDER	0x86
#define VK_CASSIOPEIA_ESC		0xC1
#define VK_CASSIOPEIA_REC		0xC2
#define VK_CASSIOPEIA_B1		0xC3
#define VK_CASSIOPEIA_B2		0xC4
#define VK_CASSIOPEIA_B3		0xC5
#define VK_CASSIOPEIA_LEFT		37
#define VK_CASSIOPEIA_UP		38
#define VK_CASSIOPEIA_RIGHT		39
#define VK_CASSIOPEIA_DOWN		40

// These are the indexes for each key into the various key arrays
#define KEY_START	0
#define KEY_EXIT	1
#define KEY_PAUSE	2
#define KEY_A		3
#define KEY_B		4
#define KEY_SELECT	5
#define KEY_LEFT	6
#define KEY_UP		7
#define KEY_RIGHT	8
#define KEY_DOWN	9
#define NUM_KEYS 10

// This array holds the actual virtual keycodes for all the keys used
int nKeyMapping[NUM_KEYS] = {
	VK_CASSIOPEIA_SLIDER,
	VK_CASSIOPEIA_ESC,
	VK_CASSIOPEIA_REC,
	VK_CASSIOPEIA_B1,
	VK_CASSIOPEIA_B2,
	VK_CASSIOPEIA_B3,
	VK_CASSIOPEIA_LEFT,
	VK_CASSIOPEIA_UP,
	VK_CASSIOPEIA_RIGHT,
	VK_CASSIOPEIA_DOWN
};

// command line arguments to send to the emulator
extern "C"
{
    extern char *argv[64];
	extern int argc;
	void PrintNonFatal(char *s);
	extern void main(int argc, char **argv);
	extern int  CESound_init(void);
	extern void CESound_exit(void);
	extern int CESound_set_volume(unsigned short left, unsigned short right);

	// color palette defined in PocketPCDriver.c
	unsigned short color_palette[256];
	
	extern void BlitScreen(void);
	int ClearScreen (int colRed,int colGreen, int colBlue);
	extern char *KeyboardGetstate(void);
	extern void errorbox(void);
	extern int pausing;
	extern int restart(void);
	extern int leftdiff = 1;
	extern int rightdiff = 0;
	extern int Touchpadx;
	extern int Touchpady;
	int tv_field=0;
}

#define MAX_LOADSTRING 100
static SHACTIVATEINFO s_sai;

void BlitScreen(void);
void OpenNewGame(HWND hWnd);
void DoPaint(HWND hWnd);
void ChangeKeyMapping(HWND hWnd, int Index);
void DrawBitmap(HDC hdc, HBITMAP hBitmap, int xStart, int yStart);

// Global Variables:
HINSTANCE			hInst;					// The current instance
HWND				hwndCB;					// The command bar handle
HBITMAP				hControllerBitmap;
HBITMAP				hConsoleBitmap;
HBITMAP				hLogoBitmap;
int Touchpadx=0;
int Touchpady=0;

// GAPI stuff
GXDisplayProperties g_gxdp;
GXKeyList g_gxkl;

// directory for roms
char gDefaultDir[2048];

// rom file name to load
char szRomFileName[2048];

// Forward declarations of functions included in this code module:
ATOM				MyRegisterClass	(HINSTANCE, LPTSTR);
int				InitInstance	(HINSTANCE, int);
LRESULT CALLBACK	WndProc			(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK	Error			(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK	Options			(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK	Console			(HWND, UINT, WPARAM, LPARAM);
extern HWND				CreateRpCommandBar(HWND);

// Process % based volume settings for CE
void Set_volume (int nOptions_SoundVolume)
{
	unsigned short left;
	unsigned short right;
	left = ((100 - nOptions_SoundVolume) * 655.35);// convert 100% to 0xffff
	right = left;
	CESound_set_volume(left,right);
}

int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	MSG msg;
	HACCEL hAccelTable;

	// Perform application initialization:
	if (!InitInstance (hInstance, nCmdShow)) 
	{
		return FALSE;
	}

	hAccelTable = LoadAccelerators(hInstance, (LPCTSTR)IDC_ATARI2600EMULATOR);

	// Main message loop:
	while (GetMessage(&msg, NULL, 0, 0)) 
	{
		if (!TranslateAccelerator(msg.hwnd, hAccelTable, &msg)) 
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		//jump into emulation if we are ready, and avoid system calls for speed.
		if (bThreadRunning )
		{
			if (nOptions_SoundOn) CESound_init();
			if (nOptions_SoundOn) Set_volume(nOptions_SoundVolume);
			main(argc,argv);
		    if (nOptions_SoundOn) CESound_exit();
		}
	}

	// clean up GAPI
	GXCloseDisplay();
	GXCloseInput();
	DeleteObject(hConsoleBitmap);
	//DeleteObject(hLogoBitmap);
	return msg.wParam;
}


//
//  FUNCTION: MyRegisterClass()
//
//  PURPOSE: Registers the window class.
//
//  COMMENTS:
//
//    It is important to call this function so that the application 
//    will get 'well formed' small icons associated with it.
//
ATOM MyRegisterClass(HINSTANCE hInstance, LPTSTR szWindowClass)
{
	WNDCLASS	wc;

    wc.style			= CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc		= (WNDPROC) WndProc;
    wc.cbClsExtra		= 0;
    wc.cbWndExtra		= 0;
    wc.hInstance		= hInstance;
    wc.hIcon			= LoadIcon(hInstance, MAKEINTRESOURCE(IDI_ATARI2600EMULATOR));
    wc.hCursor			= 0;
    wc.hbrBackground	= (HBRUSH) GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName		= 0;
    wc.lpszClassName	= szWindowClass;

	return RegisterClass(&wc);
}

//
//  FUNCTION: InitInstance(HANDLE, int)
//
//  PURPOSE: Saves instance handle and creates main window
//
//  COMMENTS:
//
//    In this function, we save the instance handle in a global variable and
//    create and display the main program window.
//
int InitInstance(HINSTANCE hInstance, int nCmdShow)
{
	HWND	hWnd = NULL;
	TCHAR	szTitle[MAX_LOADSTRING];			// The title bar text
	TCHAR	szWindowClass[MAX_LOADSTRING];		// The window class name

	hInst = hInstance;		// Store instance handle in our global variable
	// Initialize global strings
	LoadString(hInstance, IDC_ATARI2600EMULATOR, szWindowClass, MAX_LOADSTRING);
	LoadString(hInstance, IDS_APP_TITLE, szTitle, MAX_LOADSTRING);

	// load the console bitmap
	hConsoleBitmap = LoadBitmap(hInstance, MAKEINTRESOURCE(IDB_CONSOLE));
	hLogoBitmap = LoadBitmap(hInstance, MAKEINTRESOURCE(IDB_LOGO));

	//If it is already running, then focus on the window
	hWnd = FindWindow(szWindowClass, szTitle);	
	if (hWnd) 
	{
		SetForegroundWindow ((HWND) (((DWORD)hWnd) | 0x01));    
		return 0;
	} 

	MyRegisterClass(hInstance, szWindowClass);
	
	RECT	rect;
	GetClientRect(hWnd, &rect);
	
	hWnd = CreateWindow(szWindowClass, szTitle, WS_VISIBLE,
		CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, NULL, NULL, hInstance, NULL);
	if (!hWnd)
	{	
		return FALSE;
	}
	//When the main window is created using CW_USEDEFAULT the height of the menubar (if one
	// is created is not taken into account). So we resize the window after creating it
	// if a menubar is present
	{
		RECT rc;
		GetWindowRect(hWnd, &rc);
		rc.bottom -= MENU_HEIGHT;
		if (hwndCB)
			MoveWindow(hWnd, rc.left, rc.top, rc.right, rc.bottom, FALSE);
	}


	ShowWindow(hWnd, nCmdShow);
	UpdateWindow(hWnd);

	gDefaultDir[0] = 0;

	// save the handle to this window for other functions to use
	hWnd2600 = hWnd;

	// set up GAPI
	// try opening the display for full screen access
	if(GXOpenDisplay(hWnd, GX_FULLSCREEN) == 0)
		return FALSE;  // error - we can't open the display, so exit

	// initialize the hardware buttons
	if(GXOpenInput() == 0)
		return FALSE;  // error - we can't open the buttons, so exit

	// get the display properties
	g_gxdp = GXGetDisplayProperties();

	// get information about the hardware keys and fill
	// the g_gxkl structure
	g_gxkl = GXGetDefaultKeys(GX_NORMALKEYS);

	return TRUE;
}

//
//  FUNCTION: WndProc(HWND, unsigned, WORD, LONG)
//
//  PURPOSE:  Processes messages for the main window.
//
//  WM_COMMAND	- process the application menu
//  WM_PAINT	- Paint the main window
//  WM_DESTROY	- post a quit message and return
//
//
LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	int wmId, wmEvent;
	int XPos, YPos;
	switch (message) 
	{
		case WM_COMMAND:
			wmId    = LOWORD(wParam); 
			wmEvent = HIWORD(wParam); 
			// Parse the menu selections:
			switch (wmId)
			{	
				case IDM_FILE_OPEN:
					OpenNewGame(hWnd);
					break;
				case IDM_SETTINGS_OPTIONS:
					DialogBox(hInst, (LPCTSTR)IDD_OPTIONS, hWnd, (DLGPROC)Options);
					break;
				case IDM_SETTINGS_CONSOLE:
					DialogBox(hInst, (LPCTSTR)IDD_CONSOLE, hWnd, (DLGPROC)Console);
					break;
				case IDM_FILE_EXIT:
					SendMessage (hWnd, WM_CLOSE, 0, 0);
					break;
				case IDOK:
					SendMessage(hWnd, WM_ACTIVATE, MAKEWPARAM(WA_INACTIVE, 0), (LPARAM)hWnd);
					SendMessage (hWnd, WM_CLOSE, 0, 0);
					break;
				default:
				   return DefWindowProc(hWnd, message, wParam, lParam);
			}
			break;
		case WM_CREATE:
			hwndCB = CreateRpCommandBar(hWnd);
			break;
		case WM_PAINT:
			DoPaint(hWnd);
			hwndCB = CreateRpCommandBar(hWnd);
			break; 
		case WM_DESTROY:
			CommandBar_Destroy(hwndCB);
			PostQuitMessage(0);
			break;
		case WM_SETTINGCHANGE:
			SHHandleWMSettingChange(hWnd, wParam, lParam, &s_sai);
     		break;
		case WM_KEYDOWN:
			return 1;
			break;	

		case WM_KEYUP:
			return 1;
			break;

		case WM_KILLFOCUS:
			GXSuspend();
			break;

		case WM_SETFOCUS:
			GXResume();
			break;

		case WM_LBUTTONDOWN:
			break;

		case WM_MOUSEMOVE:
			if(bThreadRunning)
			{
					XPos = (unsigned int)lParam & 0xffff;
					YPos = (unsigned int)lParam >> 16;
					Touchpadx = XPos;
					Touchpady = YPos;
			}
			break;

		default:
			return DefWindowProc(hWnd, message, wParam, lParam);

   }
   return 0;
}


HWND CreateRpCommandBar(HWND hwnd)
{
	SHMENUBARINFO mbi;

	memset(&mbi, 0, sizeof(SHMENUBARINFO));
	mbi.cbSize     = sizeof(SHMENUBARINFO);
	mbi.hwndParent = hwnd;
	mbi.nToolBarId = IDM_MENU;
	mbi.hInstRes   = hInst;
	mbi.nBmpId     = 0;
	mbi.cBmpImages = 0;

	if (!SHCreateMenuBar(&mbi)) 
		return NULL;

	return mbi.hwndMB;
}

// Mesage handler for the Error box.
LRESULT CALLBACK Error(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	SHINITDLGINFO shidi;

	switch (message)
	{
		case WM_INITDIALOG:
			// Create a Done button and size it.  
			shidi.dwMask = SHIDIM_FLAGS;
		    shidi.dwFlags = SHIDIF_DONEBUTTON | SHIDIF_SIPDOWN | SHIDIF_SIZEDLGFULLSCREEN;
			shidi.hDlg = hDlg;
			SHInitDialog(&shidi);
			return TRUE; 

		case WM_COMMAND:
			if (LOWORD(wParam) == IDOK) {
				EndDialog(hDlg, LOWORD(wParam));
				return TRUE;
			}
			break;
	}
    return FALSE;
}


// Mesage handler for the Options box.
LRESULT CALLBACK Options(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	SHINITDLGINFO shidi;
	TCHAR Label[80];
	TCHAR *EndPtr;

	switch (message)
	{
		case WM_INITDIALOG:
			// Create a Done button and size it.  
			shidi.dwMask = SHIDIM_FLAGS;
			 shidi.dwFlags = SHIDIF_DONEBUTTON | SHIDIF_SIPDOWN | SHIDIF_SIZEDLGFULLSCREEN;
			shidi.hDlg = hDlg;
			SHInitDialog(&shidi);

			// put frame skip in text box
		    wsprintf (Label, TEXT("%d"), nOptions_SkipFrames);
			SendDlgItemMessage(hDlg, IDC_EDIT_SKIP_FRAMES, WM_SETTEXT, 0, (LPARAM)Label);

			//setup sound sample rate Slider control
			SendDlgItemMessage(hDlg,IDC_SND_RATE, TBM_SETRANGE, 0, (LPARAM) MAKELONG (1, 3));
			SendDlgItemMessage(hDlg,IDC_SND_RATE, TBM_SETTIC, 0, (LPARAM) MAKELONG (2,0));
			SendDlgItemMessage(hDlg,IDC_SND_RATE, TBM_SETPOS, 1, (LPARAM) MAKELONG (nOptions_SoundSampleRate , 0));

			// put interlace in text box
			SendDlgItemMessage(hDlg, IDC_EDIT_SLEEP, BM_SETCHECK, (WPARAM)nOptions_Interlace ? BST_CHECKED : BST_UNCHECKED, 0);
			// check the sound on/off button
			SendDlgItemMessage(hDlg, IDC_CHECK_SOUND, BM_SETCHECK, (WPARAM)nOptions_SoundOn ? BST_CHECKED : BST_UNCHECKED, 0);
			// put Landscape in text box
			SendDlgItemMessage(hDlg, IDC_CHECK_LAND, BM_SETCHECK, (WPARAM)nOptions_Landscape ? BST_CHECKED : BST_UNCHECKED, 0);

			//Sound Volume Slider
			SendDlgItemMessage(hDlg,IDC_VOL_SLIDER, TBM_SETRANGE, 0, (LPARAM) MAKELONG (0, 100));
			SendDlgItemMessage(hDlg,IDC_VOL_SLIDER, TBM_SETPOS, 1, (LPARAM) MAKELONG (nOptions_SoundVolume, 0));
			//Sound Buffer Size Slider
			SendDlgItemMessage(hDlg,IDC_SND_BUFFER, TBM_SETRANGE, 0, (LPARAM) MAKELONG (0, 2048));
			SendDlgItemMessage(hDlg,IDC_SND_BUFFER, TBM_SETPOS, 1, (LPARAM) MAKELONG (nOptions_SoundBufSize, 0));
			//Sound Speed Slider
			SendDlgItemMessage(hDlg,IDC_SNDSPEED_SLIDER, TBM_SETRANGE, 0, (LPARAM) MAKELONG (250, 500));
			SendDlgItemMessage(hDlg,IDC_SNDSPEED_SLIDER, TBM_SETPOS, 1, (LPARAM) MAKELONG (nOptions_SoundCreateRate, 0));

			return TRUE; 

		case WM_COMMAND:

			if (LOWORD(wParam) == IDOK) {

				//Sound Sampling Rate
				nOptions_SoundSampleRate = SendDlgItemMessage(hDlg,IDC_SND_RATE, TBM_GETPOS, 1, (LPARAM) MAKELONG (0 , 0));

				//Sound Volume Slider
				nOptions_SoundVolume = SendDlgItemMessage(hDlg,IDC_VOL_SLIDER, TBM_GETPOS, 1, (LPARAM) MAKELONG (0, 0));
	
				//Sound Buffer Size Slider
				nOptions_SoundBufSize = SendDlgItemMessage(hDlg,IDC_SND_BUFFER, TBM_GETPOS, 1, (LPARAM) MAKELONG (0, 0));

				//Sound Speed Slider
				nOptions_SoundCreateRate = SendDlgItemMessage(hDlg,IDC_SNDSPEED_SLIDER, TBM_GETPOS, 1, (LPARAM) MAKELONG (0, 0));
				
				if(SendDlgItemMessage(hDlg, IDC_EDIT_SLEEP, BM_GETCHECK, 0, 0) == BST_CHECKED)
					nOptions_Interlace = 1;
				else
					nOptions_Interlace = 0;

				if(SendDlgItemMessage(hDlg, IDC_CHECK_LAND, BM_GETCHECK, 0, 0) == BST_CHECKED)
				nOptions_Landscape = 1;
				else
				nOptions_Landscape = 0;

				if(SendDlgItemMessage(hDlg, IDC_CHECK_SOUND, BM_GETCHECK, 0, 0) == BST_CHECKED)
					nOptions_SoundOn = 1;
				else
					nOptions_SoundOn = 0;

				EndDialog(hDlg, LOWORD(wParam));
				return TRUE;
			}
			break;
	}
    return FALSE;
}

// Mesage handler for the Console box.
LRESULT CALLBACK Console(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	SHINITDLGINFO shidi;
	TCHAR Label[80];
	TCHAR *EndPtr;

	switch (message)
	{
		case WM_INITDIALOG:
			// Create a Done button and size it.  
			shidi.dwMask = SHIDIM_FLAGS;
			 shidi.dwFlags = SHIDIF_DONEBUTTON | SHIDIF_SIPDOWN | SHIDIF_SIZEDLGFULLSCREEN;
			shidi.hDlg = hDlg;
			SHInitDialog(&shidi);

			// check the Color and B/W on/off buttons
			SendDlgItemMessage(hDlg, IDC_RADIO_COLOR, BM_SETCHECK, (WPARAM)nOptions_Color ? BST_CHECKED : BST_UNCHECKED, 0);
			SendDlgItemMessage(hDlg, IDC_RADIO_BW, BM_SETCHECK, (WPARAM)(!nOptions_Color) ? BST_CHECKED : BST_UNCHECKED, 0);

			// check the P1 Difficulty on/off buttons
			SendDlgItemMessage(hDlg, IDC_RADIO_P1AM, BM_SETCHECK, (WPARAM)nOptions_P1Diff ? BST_CHECKED : BST_UNCHECKED, 0);
			SendDlgItemMessage(hDlg, IDC_RADIO_P1PRO, BM_SETCHECK, (WPARAM)(!nOptions_P1Diff) ? BST_CHECKED : BST_UNCHECKED, 0);

			// check the P2 Difficulty on/off buttons
			SendDlgItemMessage(hDlg, IDC_RADIO_P2AM, BM_SETCHECK, (WPARAM)nOptions_P2Diff? BST_CHECKED : BST_UNCHECKED, 0);
			SendDlgItemMessage(hDlg, IDC_RADIO_P2PRO, BM_SETCHECK, (WPARAM)(!nOptions_P2Diff) ? BST_CHECKED : BST_UNCHECKED, 0);

			return TRUE; 

		case WM_COMMAND:
				
			if (LOWORD(wParam) == IDC_RADIO_COLOR) 
				nOptions_Color = 1;
			else if (LOWORD(wParam) == IDC_RADIO_BW) 
				nOptions_Color = 0;
			if (LOWORD(wParam) == IDC_RADIO_P1AM) 
				nOptions_P1Diff = 1;
			else if (LOWORD(wParam) == IDC_RADIO_P1PRO) 
				nOptions_P1Diff = 0;
			if (LOWORD(wParam) == IDC_RADIO_P2AM) 
				nOptions_P2Diff = 1;
			else if (LOWORD(wParam) == IDC_RADIO_P2PRO) 
				nOptions_P2Diff = 0;

			// check the Color and B/W on/off buttons
			SendDlgItemMessage(hDlg, IDC_RADIO_COLOR, BM_SETCHECK, (WPARAM)nOptions_Color ? BST_CHECKED : BST_UNCHECKED, 0);
			SendDlgItemMessage(hDlg, IDC_RADIO_BW, BM_SETCHECK, (WPARAM)(!nOptions_Color) ? BST_CHECKED : BST_UNCHECKED, 0);

			// check the P1 Difficulty on/off buttons
			SendDlgItemMessage(hDlg, IDC_RADIO_P1AM, BM_SETCHECK, (WPARAM)nOptions_P1Diff ? BST_CHECKED : BST_UNCHECKED, 0);
			SendDlgItemMessage(hDlg, IDC_RADIO_P1PRO, BM_SETCHECK, (WPARAM)(!nOptions_P1Diff) ? BST_CHECKED : BST_UNCHECKED, 0);

			// check the P2 Difficulty on/off buttons
			SendDlgItemMessage(hDlg, IDC_RADIO_P2AM, BM_SETCHECK, (WPARAM)nOptions_P2Diff? BST_CHECKED : BST_UNCHECKED, 0);
			SendDlgItemMessage(hDlg, IDC_RADIO_P2PRO, BM_SETCHECK, (WPARAM)(!nOptions_P2Diff) ? BST_CHECKED : BST_UNCHECKED, 0);

				if (LOWORD(wParam) == IDOK) {

				EndDialog(hDlg, LOWORD(wParam));
				return TRUE;
			}
			break;
	}
    return FALSE;
}
//-----------------------------------------------------------------------------------
// This function was taken from NesCE, and modified slightly for Atari roms.
// This is called when the user selects file->open from the main menu. Display a 
// file selection dialog, listing all the nes games. If the user selectes a game,
// save the path to the rom (in a global variable - I know... shouldnt use globals),
// and start a new emulation thread.
//----------------------------------------------------------------------------------- 
void OpenNewGame(HWND hWnd)
{
	TCHAR szFileName[100];
	TCHAR szDir[100];
	DWORD rc;
	HANDLE hThread;

	TCHAR filter[] = _T("Atari Files(*.bin)\0*.bin\0All files (*.*)\0*.*\0");
	OPENFILENAME ofn;
	int i;

	// convert default rom directory from TCHAR to char
	for(i = 0; gDefaultDir[i] != 0; i++)
		szDir[i] = (TCHAR)gDefaultDir[i];
	gDefaultDir[i] = 0;

	// display a open file dialog
	memset(&ofn,0,sizeof(ofn));

	ofn.lStructSize		= sizeof(ofn);
	ofn.lpstrFilter		= filter;
	ofn.lpstrFile		= szFileName;
	ofn.nMaxFile		= 100;
	ofn.Flags			= OFN_EXPLORER | OFN_ENABLETEMPLATE;

	if(szDir[0]) 
		ofn.lpstrInitialDir = szDir;

	if(GetOpenFileName(&ofn))
	{
		// save the path to the rom file
		for(i = 0; szFileName[i] != 0; i++)
			szRomFileName[i] = (char)szFileName[i];

		szRomFileName[i] = 0;

		argc = 0;
		argv[argc++] = szRomFileName;
		argv[argc++] = szRomFileName;


		// save the new default rom directory
		for(i = 0; i < ofn.nFileOffset; i++)
			gDefaultDir[i] = (char)ofn.lpstrFile[i];
		gDefaultDir[i] = 0;

	
		// start a new emulation thread
		bThreadRunning = TRUE;

// We no longer use separate threads for the game, since it wastes time doing system calls, and the taskbar is not needed.		
/*		// start a thread to play the game
		hThread = CreateThread(NULL, 
			0,		// stack size (ignored)
			(LPTHREAD_START_ROUTINE)main,	// Thread function
			0,		// 32 bit parameter passed to thread
			0,//);//,		Creation flags (0 = start thread immediately)
			&rc);	// Receives thread ID

		CloseHandle(hThread);
*/
  }
	SetForegroundWindow(hWnd);

}


void DoPaint(HWND hWnd)
{
	HDC hdc;
	RECT rect;
	HPEN hPen, hOldPen;
	PAINTSTRUCT ps;
	
	hdc = BeginPaint(hWnd, &ps);
	if(!bThreadRunning)
	{
		DrawBitmap(hdc, hConsoleBitmap, 0, 0);
		GetClientRect(hWnd, &rect);
		rect.bottom -= 30;
		rect.top = rect.bottom - 70;

		// clear the area where the text will be drawn
		hPen = CreatePen(PS_NULL, 1, RGB(0, 0, 0));
		hOldPen = (HPEN)SelectObject(hdc, hPen);
		Rectangle(hdc, rect.left, rect.top, rect.right, rect.bottom);
		SelectObject(hdc, hOldPen);
		DeleteObject(hPen);
		DrawText(hdc, _T("PocketVCS v0.5 Atari 2600 Emulator for PocketPC - Written by Stuart Russell"), -1, &rect, DT_CENTER | DT_WORDBREAK);
			
	}

	EndPaint(hWnd, &ps);
}

void DrawBitmap(HDC hdc, HBITMAP hBitmap, int xStart, int yStart)
{
	BITMAP	bm;
	HDC		hdcMem;
	POINT	ptSize, ptOrg;

	hdcMem = CreateCompatibleDC(hdc);
	SelectObject(hdcMem, hBitmap);

	GetObject(hBitmap, sizeof(BITMAP), (LPVOID)&bm);

	ptSize.x = bm.bmWidth;
	ptSize.y = bm.bmHeight;

	ptOrg.x = 0;
	ptOrg.y = 0;

	BitBlt(hdc, xStart, yStart, ptSize.x, ptSize.y,
		hdcMem, ptOrg.x, ptOrg.y, SRCCOPY);

	DeleteDC(hdcMem);
}

void BlitScreen(void)
{
	// XBuf is the buffer with the screen data we need to blit to the screen.
	// It is arranged as scanlines of 160 bytes each, 
	// with a total of maxline scanlines of 192-222.

	extern int tv_field;
	unsigned char *pbuf = XBuf;
	unsigned char *pbuf2 = XBuf;
	int xx, yy;
	static int nPadDisplay = (g_gxdp.cbyPitch >> 1) - 240;//this is pad space after the visible area
	int zz=0;

	
	if (nOptions_Interlace==1) pbuf2+=(160*222);
	tv_field = !tv_field;
	// get a pointer to the screen
	unsigned short *pscr = (unsigned short *)GXBeginDraw();
	if(pscr == NULL)
		return;
	
	if (!nOptions_Landscape){
		// We are in Portrait mode...
		// start drawing 32 lines from top of display
		pscr += (240 + nPadDisplay) << 5;

			for(yy = 0; yy < 210; yy++)
			{
				for(xx = 0; xx < 160; xx++)
				{
					*pscr = color_palette[*pbuf] | color_palette[*pbuf2];
					pscr++;
					// This next section converts 160 pixels to 240 for full width display.
						zz=(zz==FALSE);				
					if(zz){
						*pscr = color_palette[*pbuf] | color_palette[*pbuf2];
						pscr++;}
					pbuf++;
					pbuf2++;
				}
				pscr += nPadDisplay;
			}
	}else{//We are in Landscape mode...
		pscr += 15;//start down halfway (actually X Pixels)
			for(xx = 0; xx < 210; xx++)
			{
				pscr += (240 + nPadDisplay) *320;//start in bottem left
				pscr ++;
				for(yy = 0; yy < 160; yy++)
				{
					*pscr = color_palette[*pbuf] | color_palette[*pbuf2];
					pscr-= (240 + nPadDisplay);//move back one Y pixel
					*pscr = color_palette[*pbuf] | color_palette[*pbuf2];
					pscr-= (240 + nPadDisplay);//move back one Y pixel
					pbuf++;
					pbuf2++;
				}
			}
	}

	GXEndDraw();
	
}

char KeyboardBuffer[256];

//------------------------------------------------------
// initialize the keyboard (buttons) on program start
//------------------------------------------------------
int KeyboardInitialize(void)
{
	int i;

	for(i = 0; i < 256; i++)
		KeyboardBuffer[i] = 0;

	// return 1 for success, 0 for failure
	return 1;
}

//----------------------------------------------------------------------------------
// This is also from NesCE, with some modifications...
// KeyboardGetState reads the state of the buttons, and writes it to the char
// array KeyboardBuffer. KeyboardBuffer is large enough to support 256 keys. If 
// a key is pressed, the cooresponding byte is set to 0x80. If it is not pressed, 
// the byte is set to 0x00 in the array. See Keyscan.h for constant indexes to use.
//----------------------------------------------------------------------------------
char *KeyboardGetstate(void)
{
	
if (!nOptions_Landscape){
	KeyboardBuffer[SCANCODE_CURSORUP] = GetAsyncKeyState(nKeyMapping[KEY_UP]);
	KeyboardBuffer[SCANCODE_CURSORDOWN] = GetAsyncKeyState(nKeyMapping[KEY_DOWN]);
	KeyboardBuffer[SCANCODE_CURSORLEFT] = GetAsyncKeyState(nKeyMapping[KEY_LEFT]);
	KeyboardBuffer[SCANCODE_CURSORRIGHT] = GetAsyncKeyState(nKeyMapping[KEY_RIGHT]);
}else{
	KeyboardBuffer[SCANCODE_CURSORUP] = GetAsyncKeyState(nKeyMapping[KEY_LEFT]);
	KeyboardBuffer[SCANCODE_CURSORDOWN] = GetAsyncKeyState(nKeyMapping[KEY_RIGHT]);
	KeyboardBuffer[SCANCODE_CURSORLEFT] = GetAsyncKeyState(nKeyMapping[KEY_DOWN]);
	KeyboardBuffer[SCANCODE_CURSORRIGHT] = GetAsyncKeyState(nKeyMapping[KEY_UP]);
}
	KeyboardBuffer[SCANCODE_LEFTALT] = GetAsyncKeyState(nKeyMapping[KEY_A]);
	KeyboardBuffer[SCANCODE_LEFTCONTROL] = GetAsyncKeyState(nKeyMapping[KEY_B]);
	KeyboardBuffer[SCANCODE_TAB] = GetAsyncKeyState(nKeyMapping[KEY_SELECT]);
	KeyboardBuffer[SCANCODE_ENTER] = GetAsyncKeyState(nKeyMapping[KEY_START]);
	KeyboardBuffer[SCANCODE_ESCAPE] = GetAsyncKeyState(nKeyMapping[KEY_EXIT]);
			
	return KeyboardBuffer;
}

void errorbox()
{
	DialogBox(hInst, (LPCTSTR)IDD_Error, 0, (DLGPROC)Error);
	pausing = 1;
}

//This routine was taken right from Microsofts website, pretty obvious use..:)
int ClearScreen(int colRed,int colGreen, int colBlue)
{
	
						// 16 bit per pixel code.  Note 2 different pixel formats.
                          switch (g_gxdp.cBPP)
                          {
                          case 16: {
                                      unsigned short PixelCol = 0;
                                      if (g_gxdp.ffFormat & kfDirect565) {
                                            PixelCol = (unsigned short) 
                                                        ((colRed & 0xff)<< 11 | 
                                                         (colGreen & 0xff) << 6 | 
                                                         (colBlue & 0xff));
                                      } else if (g_gxdp.ffFormat & kfDirect555) {
                                            PixelCol = (unsigned short) 
                                                        ((colRed & 0xff)<< 10 | 
                                                         (colGreen & 0xff) << 5 | 
                                                         (colBlue & 0xff));
                                      }
                                
                         unsigned short * pusLine = (unsigned short *)GXBeginDraw();
                         if (pusLine == NULL) return false; // NOT OK TO DRAW
                          
                             for (unsigned int y = 0; y < g_gxdp.cyHeight; y++) {
                                   unsigned short * pusDest = pusLine;
                                   for (unsigned int x = 0;
                                        x < g_gxdp.cxWidth; x++) {
                                         *pusDest = PixelCol;
                                         pusDest += g_gxdp.cbxPitch >> 1;
                                   }
                                   pusLine += g_gxdp.cbyPitch >> 1;
                             }
                             GXEndDraw();
                             break;
                             }
                            case 8:         {
                                  // Get a Random Color
                                  // IMPORTANT:
                                  // 8Bit are using a palette and the formular
                                  // below does not compute a read RGB color
                                  unsigned char bPixel = (colRed &0xf)<<5 |
                                                          (colGreen&0x3)<<3 |
                                                          (colBlue&0xF);
                                                          
                            unsigned char * pbLine = (unsigned char *)GXBeginDraw();
                            if (pbLine == NULL)     return false;// NOT OK TO DRAW
                            
                               for (unsigned int y = 0; y < g_gxdp.cyHeight; y++) {
                                     unsigned char * pbDest = pbLine;
                                     for (unsigned int x = 0;
                                          x < g_gxdp.cxWidth; x++) {
                                           *pbDest = bPixel;
                                           pbDest += g_gxdp.cbxPitch;
                                     }
                                     pbLine += g_gxdp.cbyPitch;
                               }
                               GXEndDraw();
                               break;
                               }
                         }
			//Added for controller bitmap..NOT READY
			/*HDC hdc;
			PAINTSTRUCT ps;
			hdc = BeginPaint(hWnd2600, &ps);
			DrawBitmap(hdc , hLogoBitmap, 0, 215);
			*/
			return false;
			
  }
